/* 
  Rui Santos
  Complete project details at https://RandomNerdTutorials.com/esp8266-nodemcu-door-status-telegram/
  
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files.
  
  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.
  updated by Riccardo Grosso, 22/04/2024. https://www.ethicaldiy.org
*/

#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>
#include <UniversalTelegramBot.h>
#include <ArduinoJson.h>

// Set GPIOs for  Drled and DrReedSwitch
const int DrReedSwitch = 2;   //Door switch state
//const int  Drled = 2; //optional
const int WnReedSwitch = 0;   //Window switch state
//const int  Wnled = 13; //optional


// Detects whenever the door changed Drstate
bool changeDrState = false;
bool changeWnState = false;

// Holds DrReedSwitch Drstate (1=opened, 0=close)
bool Drstate;
String doorState;
bool Wnstate;
String winState;

// Auxiliary variables (it will only detect changes that are 1500 milliseconds apart)
unsigned long previousMillis = 0; 
const long interval = 1500;

const char* ssid = "Inserisci la SSID della tua rete WiFi";
const char* password = "Inserisci la tua password";

// Initialize Telegram BOT
#define BOTtoken "Inserisci qui il tuo token API"   // your Bot Token (Get from Botfather)

// Use @myidbot to find out the chat ID of an individual or a group
// Also note that you need to click "start" on a bot before it can
// message you
#define CHAT_ID "Inserisci qui il tuo userID"

X509List cert(TELEGRAM_CERTIFICATE_ROOT);
WiFiClientSecure client;
UniversalTelegramBot bot(BOTtoken, client);

// Runs whenever the DrReedSwitch changes Drstate
ICACHE_RAM_ATTR void changeDoorStatus() {
  Serial.println("State Door changed");
  changeDrState = true;
  }
ICACHE_RAM_ATTR void changeWinStatus() {
  Serial.println("State Window changed");
  changeWnState = true;
}

void setup() {
  // Serial port for debugging purposes
  Serial.begin(115200);
  configTime(0, 0, "pool.ntp.org");      // get UTC time via NTP
  client.setTrustAnchors(&cert); // Add root certificate for api.telegram.org
  

  // Read the current door Drstate
  pinMode(DrReedSwitch, INPUT_PULLUP);
  Drstate = digitalRead(DrReedSwitch);
  pinMode(WnReedSwitch, INPUT_PULLUP);
  Wnstate = digitalRead(WnReedSwitch);

  // Set  Drled Drstate to match door Drstate
  //pinMode( Drled, OUTPUT);
  //digitalWrite( Drled, Drstate);
  //pinMode( Wnled, OUTPUT);
  //  digitalWrite( Wnled, Wnstate);
  
  // Set the DrReedSwitch pin as interrupt, assign interrupt function and set CHANGE mode
  attachInterrupt(digitalPinToInterrupt(DrReedSwitch), changeDoorStatus, CHANGE);
  attachInterrupt(digitalPinToInterrupt(WnReedSwitch), changeWinStatus, CHANGE);

  // Connect to Wi-Fi
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
    while (WiFi.status() != WL_CONNECTED) {
      delay(500);
      Serial.print(".");
    }
    Serial.println("");
  Serial.print("\nWiFi connected. IP address: ");
  Serial.println(WiFi.localIP());

    bot.sendMessage(CHAT_ID, "Bot started up", "");
}

void loop() {
  if (changeDrState){
    unsigned long currentMillis = millis();
    if(currentMillis - previousMillis >= interval) {
      previousMillis = currentMillis;
      // If a Drstate has occured, invert the current door Drstate   
        Drstate = !Drstate;
        if(Drstate) {
          doorState = "closed";
        }
        else{
          doorState = "open";
        }
        //digitalWrite(Drled, !Drstate);
        changeDrState = false;
        Serial.println(Drstate);
        Serial.println(doorState);
        
        //Send notification
        bot.sendMessage(CHAT_ID, "The door is " + doorState, "");
    }  
  }
   if (changeWnState){
    unsigned long currentMillis = millis();
    if(currentMillis - previousMillis >= interval) {
      previousMillis = currentMillis;
      // If a Drstate has occured, invert the current door Drstate   
        Wnstate = !Wnstate;
        if(Wnstate) {
          winState = "closed";
        }
        else{
          winState = "open";
        }
        //digitalWrite(Wnled, !Wnstate);
        changeWnState = false;
        Serial.println(Wnstate);
        Serial.println(winState);
        
        //Send notification
        bot.sendMessage(CHAT_ID, "The window is " + winState, "");
    } 
   } 
}
